package es.rickyepoderi.springboot.rest;

import java.io.IOException;
import java.lang.reflect.Method;
import javax.annotation.security.DenyAll;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.container.ResourceInfo;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;

/**
 * https://howtodoinjava.com/jersey/jersey-rest-security/
 * 
 * @author rmartinc
 */
public class RolesAuthorizationFilter implements ContainerRequestFilter {

    @Context
    private ResourceInfo resourceInfo;

    @Context
    private HttpServletRequest httpServletRequest;

    private void notAllowed(ContainerRequestContext ctx) {
        ctx.abortWith(Response.status(Response.Status.FORBIDDEN).build());
    }

    private void checkRoles(ContainerRequestContext ctx, String[] rolesAllowed) {
        if (rolesAllowed != null) {
            for (String role : rolesAllowed) {
                if (httpServletRequest.isUserInRole(role)) {
                    // user is allowed
                    return;
                }
            }
        }
        // user is not of allowed roles => deny
        notAllowed(ctx);
    }

    @Override
    public void filter(ContainerRequestContext ctx) throws IOException {
        // look for method/class annotations to allow access
        Method method = resourceInfo.getResourceMethod();
        Class clazz = method.getDeclaringClass();
        if (method.isAnnotationPresent(DenyAll.class)) {
            // deny at method level
            notAllowed(ctx);
        } else if (method.isAnnotationPresent(PermitAll.class)) {
            // allowed at method level => ok
        } else if (method.isAnnotationPresent(RolesAllowed.class)) {
            // check roles allowed
            checkRoles(ctx, ((RolesAllowed) method.getAnnotation(RolesAllowed.class)).value());
        } else if (clazz.isAnnotationPresent(DenyAll.class)) {
            // deny at class level
            notAllowed(ctx);
        } else if (clazz.isAnnotationPresent(PermitAll.class)) {
            // allowed at class level => ok
        } else if (clazz.isAnnotationPresent(RolesAllowed.class)) {
            // check roles allowed in the class
            checkRoles(ctx, ((RolesAllowed) clazz.getAnnotation(RolesAllowed.class)).value());
        }
        // no annotation is like PermitAll => allowed
    }
}
